

#include <stdio.h>
#include <NIDAQmx.h>
#include <Windows.h>
#include "lsl_c.h"

#define DAQmxErrChk(functionCall) if( DAQmxFailed(error=(functionCall)) ) goto Error; else

int main(void)
{
	int			error=0;
	TaskHandle	taskHandle=0,taskHandleIn=0;
	char		errBuff[2048]={'\0'};
	float64     data[1] = {5.0};
	float64		dataIn[1] = {0.0};
//	float dataInFloat[1] = {0.0};
	int i=0;
	int count;
	int32 read;
	char *markertypes[] = {"/Experiment Control/Synchronization/Trigger/Biosemi"};

	lsl_streaminfo info;		/* out stream declaration object */
	lsl_outlet outlet;			/* stream outlet */
	//lsl_outlet dataOutlet;

	/* declare a new streaminfo (name: "MyEventStream", content type: "Markers", 1 channel, irregular rate, ... */
	/* ... string values, some made-up source id (can also be empty) */
	info = lsl_create_streaminfo("SynchronizationBiosemi","Markers",1,LSL_IRREGULAR_RATE,cft_string,"bf37e271-e0ee-455b-bc9e-6c23a3308d0e");

	/* make a new outlet (chunking: default, buffering: 360k markers) */
	outlet = lsl_create_outlet(info,0,360);

	//info = lsl_create_streaminfo("Heartbeat", "Heartbeat", 1, LSL_IRREGULAR_RATE, cft_float32, "afadfjd8f7sf");
	//dataOutlet = lsl_create_outlet(info,0,360);
//	printf("Press Enter to begin.\n");
//	getchar();
	printf("Now sending markers...\n");

	/*********************************************/
	// DAQmx Configure Code
	/*********************************************/
	DAQmxErrChk (DAQmxCreateTask("",&taskHandle));
	DAQmxErrChk (DAQmxCreateAOVoltageChan(taskHandle,"Dev1/ao0","",-10.0,10.0,DAQmx_Val_Volts,""));
	DAQmxErrChk (DAQmxCfgSampClkTiming(taskHandle,"OnboardClock",10000.0,DAQmx_Val_Rising,DAQmx_Val_HWTimedSinglePoint,1));

	DAQmxErrChk (DAQmxCreateTask("",&taskHandleIn));
	DAQmxErrChk (DAQmxCreateAIVoltageChan(taskHandleIn,"Dev1/ai0","",DAQmx_Val_RSE,-10.0,10.0,DAQmx_Val_Volts,NULL));
	DAQmxErrChk (DAQmxCfgSampClkTiming(taskHandleIn,"OnboardClock",10000.0,DAQmx_Val_Falling,DAQmx_Val_HWTimedSinglePoint,1));

	//occasionally a clock will be missed. This prevents errors from being thrown on NI-6321.
	//order of events is still preserved, by the nature of DAQmx_HWTimedSinglePoint
	DAQmxErrChk (DAQmxSetRealTimeConvLateErrorsToWarnings(taskHandle, 1));
	/*********************************************/
	// DAQmx Start Code
	/*********************************************/
	DAQmxErrChk (DAQmxStartTask(taskHandle));
	DAQmxErrChk (DAQmxStartTask(taskHandleIn));

	
	/*********************************************/
	// DAQmx Write and Read Code
	/*********************************************/
	for(i=0; /*i<3600*/; i++) {
		data[0] = 5.0;
		

		
		count = 0;
		do {
		 DAQmxErrChk (DAQmxWriteAnalogF64(taskHandle,1,1,10.0,DAQmx_Val_GroupByChannel,data,NULL,NULL));
		 DAQmxErrChk (DAQmxReadAnalogF64(taskHandleIn,1,10.0,DAQmx_Val_GroupByChannel,dataIn,1,&read,NULL));

		 count++;
		} while (dataIn[0] < 2.5);

		 lsl_push_sample_str(outlet,&(markertypes[0]));


		//finish out the 3000 us pulse
		while (count < 30) {
			DAQmxErrChk (DAQmxWriteAnalogF64(taskHandle,1,1,10.0,DAQmx_Val_GroupByChannel,data,NULL,NULL));
			DAQmxErrChk (DAQmxReadAnalogF64(taskHandleIn,1,10.0,DAQmx_Val_GroupByChannel,dataIn,1,&read,NULL));

			count++;
		}

		data[0] = 0.0;

		do {
			DAQmxErrChk (DAQmxWriteAnalogF64(taskHandle,1,1,10.0,DAQmx_Val_GroupByChannel,data,NULL,NULL));
			DAQmxErrChk (DAQmxReadAnalogF64(taskHandleIn,1,10.0,DAQmx_Val_GroupByChannel,dataIn,1,&read,NULL));

			count++;
		} while (dataIn[0] > 2.5);

		while (count <  10000) {

			DAQmxErrChk (DAQmxWriteAnalogF64(taskHandle,1,1,10.0,DAQmx_Val_GroupByChannel,data,NULL,NULL));
			DAQmxErrChk (DAQmxReadAnalogF64(taskHandleIn,1,10.0,DAQmx_Val_GroupByChannel,dataIn,1,&read,NULL));
		//	 DAQmxErrChk (DAQmxWaitForNextSampleClock (taskHandle, 1000.0, &isLate));

			count++;
		}
	//	Sleep(1000);
		//printf("Marker: %d\n", i);

	}
Error:
	if( DAQmxFailed(error) )
		DAQmxGetExtendedErrorInfo(errBuff,2048);
	if( taskHandle!=0 ) {
		/*********************************************/
		// DAQmx Stop Code
		/*********************************************/
		DAQmxStopTask(taskHandle);
		DAQmxClearTask(taskHandle);
	}
	if(taskHandleIn!=0) {
		/*********************************************/
		// DAQmx Stop Code
		/*********************************************/
		DAQmxStopTask(taskHandleIn);
		DAQmxClearTask(taskHandleIn);

	}
	if( DAQmxFailed(error) )
		printf("DAQmx Error: %s\n",errBuff);
	printf("End of program, press Enter key to quit\n");
	getchar();
	return 0;
}
